"""
Admin interface is following a customization pattern explain in Django Admin docs. 
The implemented hierarchy is as follows:
::
  -> class ProfileAdmin(ReadPermissionModelAdmin) uses ProfileForm & MetricInstanceInline
  \-->  class ProfileForm(forms.ModelForm)
  \-->  class MetricInstanceInline(admin.TabularInline) uses MetricInstanceForm
    \---> class MetricInstanceForm(forms.ModelForm)
    
Read-only ModelAdmin extension is documented in :py:mod:`poem.admin_ext`. Custom ProfileForm connects
core profile attribute to jQuery widgets. MetricInstanceForm does the same for metric instances. 

"""
from django import forms
from django.forms import ValidationError
from django.contrib.auth.models import User
from django.contrib.auth.admin import UserAdmin
from django.contrib import admin
from django.conf import settings

from piston.models import Nonce, Consumer, Token, Resource
from taggit.forms import TagField

from Poem.poem.models import MetricInstance, Profile, UserProfile
from Poem.poem import widgets
from Poem.poem.admin_ext import ReadPermissionModelAdmin
from Poem.poem_api.networking import get_hints 

HINTS_URL = u"'%s" % (settings.POEM_URL_PREFIX+"/api/0.1/json/hints")

class MetricInstanceForm(forms.ModelForm):
    """
    Connects metric instance attributes to autocomplete widget (:py:mod:`poem.widgets`).
    """
    class Meta:
        model = MetricInstance
        exclude = ('vo',)
    
    metric = forms.CharField(help_text='Namespace of the probe.', 
                             label='Metric', max_length=128,
                             widget = widgets.JQueryAutoComplete(
                                        options={'source': HINTS_URL+"/metrics/'",
                                                 'minLegth': 2},
                                        attrs={'maxlength': 128, 'size': 50})
                            )
    atp_service_type_flavour = forms.CharField(label='Service Flavour', 
                                help_text='Service flavour of the probe.', max_length=128,
                                widget = widgets.JQueryAutoComplete(
                                  options={'source': HINTS_URL+"/atp_service_type_flavour/'",
                                           'minLength': 2},
                                  attrs={'maxlength': 128, 'size': 25})
                                )
#    vo = forms.CharField(help_text='Virtual organization.', required=False,
#                             label='VO', max_length=128,
#                             widget = widgets.JQueryAutoComplete(
#                                            options={'source': HINTS_URL+"/atp_vo/'",
#                                                     'minLength': 2},
#                                            attrs={'maxlength': 128, 'size': 11}))
    
    def clean_atp_service_type_flavour(self):
        form_flavour = self.cleaned_data['atp_service_type_flavour']
        (status, clean_values) = get_hints('atp_service_type_flavour')
        if status and form_flavour not in clean_values:
            raise ValidationError("Unable to find flavour %s in ATP." % (str(form_flavour)))
            
        return form_flavour 
      
class MetricInstanceInline(admin.TabularInline):
    model = MetricInstance
    form = MetricInstanceForm
        
class ProfileForm(forms.ModelForm):
    """
    Connects profile attributes to autocomplete widget (:py:mod:`poem.widgets`). Also
    adds media and does basic sanity checking for input.
    """
    class Meta:
        model = Profile
        
    class Media:
        css = { "all" : ("/poem_media/css/poem_profile.custom.css",) }
        js = ( '/poem_media/js/poem-custom.js', )

    name = forms.CharField(help_text='Namespace and name of this profile. This entry will appear in the MyEGI/MyWLCG.', 
                           max_length=128,
                           widget = widgets.NamespaceTextInput(
                                           attrs={'maxlength': 128, 'size': 45})
                           )
    atp_vo = forms.CharField(help_text='Virtual organization that owns this profile.',
                             label='VO', max_length=128,
                             widget = widgets.JQueryAutoComplete(
                                            options={'source': HINTS_URL+"/atp_vo/'",
                                                     'minLength': 2},
                                            attrs={'maxlength': 128, 'size': 11})
                             )
    tags = TagField(required=False, help_text='A comma separated list of tags assigned to this profile.',
                           label='Tags', 
                           max_length=128,
                           widget = widgets.TagAutocomplete(attrs={'style':'width:480px',
                                                                   'autocomplete': 'off'})
                           )
    description = forms.CharField(help_text='Free text description outlining the purpose of this profile.',
                                  widget = forms.Textarea(attrs={'style':'width:480px;height:100px'})
                                  )
    owner = forms.CharField(required=False,
                            help_text='Certificate DN of the owner of this profile (if present it implies authorization control for changes).',
                            widget=forms.TextInput(attrs={'style':'width:540px'}))
    
    def clean_atp_vo(self):
        form_vo = self.cleaned_data['atp_vo']
        (status, clean_values) = get_hints('atp_vo')
        if status and form_vo not in clean_values:
            raise ValidationError("Unable to find virtual organization %s in ATP." % (str(form_vo)))
        
        return form_vo
    
    def clean(self):
        # Basic object-level authorization workaround
        super(ProfileForm, self).clean()
        
        owner = self.instance.owner
        if self._request.user.is_superuser:
            return self.cleaned_data
        
        # if user has no profile (default django user) then deny
        try:
            self._request.user.get_profile()
        except UserProfile.DoesNotExist:
            raise ValidationError("Unable to authorize user, please request to add your DN to this profile (%s)." % owner)
        
        if owner and self._request.user.get_profile().subject != owner:
            raise ValidationError("Sorry, this profile can only be changed by its owner (%s)." % owner)
        
        return self.cleaned_data
    
class ProfileAdmin(ReadPermissionModelAdmin):
    """
    POEM admin core class that customizes its look and feel. 
    """
    list_display = ('name', 'atp_vo', 'formatted_tags', 'description')
    list_filter = ('atp_vo', 'tags')
    search_fields = ('name', 'atp_vo',)
    fields = ('name', 'atp_vo', 'owner', 'tags', 'description')
    #filter_vertical = ('groups', 'metric_instances')
    inlines = [ MetricInstanceInline,  ]
    exclude = ('version', 'valid_from', 'valid_to', 'valid', 'extensions' )
    form = ProfileForm 
    
    def get_form(self, request, obj=None, **kwargs):
        cls = super(ProfileAdmin, self).get_form(request, obj=None, **kwargs)
        cls._request = request
        return cls
    
    def save_model(self, request, obj, form, change):
        obj.save()
        if obj.groups.filter(is_deleted='Y'):
            obj.valid = False
        else:
            obj.valid = True
        obj.save()
        
    def get_row_css(self, obj, index):
        if not obj.valid:
            return 'row_red red%d' % index
        return ''
    
admin.site.register(Profile, ProfileAdmin)

admin.site.unregister(User)

class UserProfileForm(forms.ModelForm):
    subject = forms.CharField(widget=forms.TextInput(attrs={'style':'width:500px'}))

class UserProfileInline(admin.StackedInline):
    model = UserProfile
    form = UserProfileForm

class UserProfileAdmin(UserAdmin):
    inlines = [UserProfileInline]

admin.site.register(User, UserProfileAdmin)

admin.site.unregister(Consumer) 
admin.site.unregister(Nonce) 
admin.site.unregister(Token) 
admin.site.unregister(Resource)  

