"""
Profile Management Web API is implemented using 
`Django Piston <https://bitbucket.org/jespern/django-piston/wiki/Documentation>`_ 
and follows its conventions. The implementation has two set of piston handlers, the first one 
generates the output for POEM models (via Django ORM):
 * :py:class:`poem_api.handlers.NamespaceHandler`
 * :py:class:`poem_api.handlers.ProfileHandler` 
 * :py:class:`poem_api.handlers.MetricInstanceHandler` 

This group is accessed by poem-sync to synchronize profiles to a local instance.
The second group implements custom resources to provide support for jQuery widgets (e.g. autocomplete): 
 * :py:class:`poem_api.handlers.SuggestionHandler`
 * :py:class:`poem_api.handlers.MetricHandler`
 * :py:class:`poem_api.handlers.TagsHandler`

This group is used to implement AJAX calls for POEM Admin widgets.
Current API is read-only as only GET method calls are supported. 

.. note::

    Please note that internal API can change without any prior notice and only 
    supports internal SAM components.

"""

from django.http import HttpResponse, HttpResponseNotFound,\
    HttpResponseBadRequest, HttpResponseServerError
from django.conf import settings
from django.core.cache import cache
from django.utils.datastructures import MultiValueDictKeyError
from django.utils import simplejson
from django.core.serializers.json import DateTimeAwareJSONEncoder

from piston import handler
from piston.emitters import Emitter
from piston.handler import typemapper
from taggit.models import Tag

from Poem_sync.poem_sync.models import MetricMapping

from Poem.poem import models   
from Poem.poem_api import networking


class NamespaceHandler(handler.BaseHandler):
    """ 
    Lists the namespace for which the POEM Web instance was registered. A set of 
    namespaces that can be synchronized needs to be unique. Therefore it is a 
    good practice to add domain to your namespace. Sample output:
    ::
        { "namespace": "ch.cern.sam" }

    * *Configuration*: POEM_NAMESPACE yaim variable.
    * *URL*: ``/api/0.1/<format>/namespace`` 
    * *Supported formats*: xml, json
    * *Supported methods*: GET
    """
    methods_allowed = ('GET',)

    def read(self, request):
        return { 'namespace': settings.POEM_NAMESPACE }
    
 
class ProfileHandler(handler.BaseHandler):
    """ 
    Dumps the list of profiles available in this namespace. This API call is used by the
    poem-sync synchronizer to get list of profiles and their attributes. Sample output:
    ::
        [ {
        "name": "ALICE",
        "atp_vo": "alice",
        "metric_instances": [
            {
                "atp_service_type_flavour": "CE",
                "fqan": "",
                "metric": "org.sam.CE-JobSubmit",
                "vo": "alice"
            }, ....
        ] ]
 
    * *Supported formats*: xml, json
    * *URL*: ``/api/0.1/<format>/profiles``
    * *Supported methods*: GET
    """
    model = models.Profile
    allowed_methods = ('GET')
    fields = ('id', 'name', 'version', 'atp_vo', 
              'valid_from', 'valid_to', 'owner', 'description',
              ('formatted_tags'),
              ('groups', ('atp_groupname', 'atp_grouptype')),
              ('metric_instances', ('metric', 'fqan', 'vo', 'atp_service_type_flavour')),
              ('extensions')
             )
    
class CompProfileHandler(ProfileHandler):
    """
    Compatibility profile handler that enhances regular profile handler with 
    metric and service type mapping. This is used to support backward compa-
    tibility for one to one replacements of metrics and service types. Proper
    way of handling metric replacements needs to be implemented in the future.
    """
    
    METRIC_MAP = MetricMapping.objects.all().values_list('new_metric', 'old_metric')
        
    def _mi_map(self, profiles, metric_index):
        """
        Replaces metrics using corresponding index.
        """
        for profile in profiles:
            for mi in profile['metric_instances']:
                if mi['metric'] in metric_index.keys():
                    mi['metric'] = metric_index[mi['metric']]
        return profiles
    
    def read(self, request, *args, **kwargs):
        """
        Overloading method and adding some emitter magic since Piston doesn't
        support well the possibility to modify read output. This only works
        for JSON now, but it can be extended to other formats (via 
        resource.determine_emitter). 
        """
        qs = super(CompProfileHandler,self).read(request, *args, **kwargs)

        try:        
            metric_index = dict(self.METRIC_MAP)
        except IndexError:
            return HttpResponseServerError("Failed to parse metric or service type map.")
        
        emitter, _ = Emitter.get("json")
        et = emitter(qs, typemapper, self, self.fields, None)
        
        et_result = et.construct()

        result = self._mi_map(et_result, metric_index)
        
        srl = simplejson.dumps(result, cls=DateTimeAwareJSONEncoder, ensure_ascii=False, indent=4)    
        return HttpResponse(srl, mimetype='application/json', status=200)
        

class MetricInstanceHandler(handler.BaseHandler):
    """ 
    Lists all unique metric instances registered in the DB. Metric instance is a tuple
    ``('metric', 'atp_service_type_flavour', 'vo', 'fqan')``. This information is used
    by the poem synchronizer. Sample output:
    ::
        [ {
        "metric": "org.apel.APEL-Pub",
        "fqan": "",
        "atp_service_type_flavour": "APEL",
        "vo": "ops"
          }, ...
        ]

    * *URL*: ``/api/0.1/<format>/metricinstances`` 
    * *Supported formats*: xml, json
    * *Supported methods*: GET
    """

    def read(self, request):
        result = models.MetricInstance.objects.values('metric', 
                                                      'atp_service_type_flavour', 'vo', 'fqan').distinct()
        return HttpResponse(simplejson.JSONEncoder().encode(list(result)),
                                mimetype='application/json')
        
    
class MetricHandler(handler.BaseHandler):
    """ 
    Helper resource to lookup metrics based on a term. This is used by the autocomplete
    widgets in Django Admin. For a sample call
    ``/api/0.1/json/hints/metrics/?term=CE`` the following output is provided:
    ::
        [
        "org.sam.CREAMCE-JobState",
        "org.sam.CREAMCE-DirectJobState",
        "org.sam.mpi.CE-JobSubmit", ...
        ]
        
    * *URL*: ``/api/0.1/json/hints/metrics/`` 
    * *Supported formats*: json
    * *Supported methods*: GET
    """
    def read(self, request):
        if 'term' in request.GET:
            term = request.GET.get('term')
            qs = models.MetricInstance.objects.filter(metric__contains=term).values_list('metric').distinct()
            result = set(map(lambda x: x[0], qs))
            return HttpResponse(simplejson.JSONEncoder().encode(list(result)),
                                mimetype='application/json')
        else:
            qs = models.MetricInstance.objects.all().values_list('metric').distinct()
            result = set(map(lambda x: x[0], qs))
            return HttpResponse(simplejson.JSONEncoder().encode(list(result)),
                                mimetype='application/json')
        
class GroupReferenceHandler(handler.BaseHandler):
    model = models.GroupReference
    allowed_methods = ('GET')
    
class ExtensionHandler(handler.BaseHandler):
    model = models.Extension
    allowed_methods = ('GET')
    
class ExpressionHandler(handler.BaseHandler):
    model = models.AvailabilityExpression
    allowed_methods = ('GET')
    
class SuggestionHandler(handler.BaseHandler):
    """ 
    Helper resource to lookup ATP attributes (e.g. VO, flavours) by calling ATP API. This is used by the autocomplete
    widgets in Django Admin. For a sample call
    ``/api/0.1/json/hints/atp_vo/?term=alice`` the following output is provided:
    ::
        [ "calice", "alice" ]
        
    * *Configuration*: ATP API mapping for given attribute is configured in settings via
                               SUGGESTION_URIS and SUGGESTION_KEYS
    * *URL*: ``/api/0.1/json/hints/<atp_attribute>/?term=<lookup_term>`` 
    * *Supported formats*: json
    * *Supported methods*: GET
    """
    def read(self, request, attribute):
        # term=lookup_query
        # attribute to parse in json feed
        if attribute not in settings.SUGGESTION_URIS:
            return HttpResponseNotFound('Configuration parameter missing (SUGGESTION_URIS)')
        
        if attribute not in settings.SUGGESTION_KEYS:
            return HttpResponseNotFound('Configuration parameter missing (SUGGESTION_KEYS)')
        
        if 'term' not in request.GET:
            return HttpResponseBadRequest
        
        cache_key = request.path.split('?')[0]
        lookup = request.GET.get('term')
        
        if not cache.get(cache_key):
            # if not in cache
            (status, json_values) = networking.get_hints(attribute)
            if status:
                cache.set(cache_key, json_values)
            else:
                return HttpResponseServerError
        else:
            # if in cache
            json_values = cache.get(cache_key)

        result = filter(lambda x: lookup in x, json_values)
        return  HttpResponse(simplejson.JSONEncoder().encode(result), 
                                mimetype='application/json')

class TagsHandler(handler.BaseHandler):
    """ 
    .. versionadded:: SAM-Update 19
    Helper resource to lookup existing tags. This is used by the autocomplete
    widgets in Django Admin. For a sample call
    ``/api/0.1/json/tags/?term=nagi`` the following output is provided:
    ::
        [ {
        "id": 28,
        "value": "nagios",
        "slug": "nagios"
        }, ... ]
        
    * *URL*: ``/api/0.1/json/tags/?term=<lookup_term>`` 
    * *Supported formats*: json
    * *Supported methods*: GET
    """
    def read(self, request):
        try:
            tags = [ {'id': tag.id, 'slug':tag.slug, 'value': tag.name} for tag in 
                                            Tag.objects.filter(name__istartswith=request.GET['term']) ]
        except MultiValueDictKeyError:
            return HttpResponseNotFound('MultiValueDictKeyError caught while processing the request.')

        return HttpResponse(simplejson.dumps(tags), mimetype='application/json')
  